/**
 * David's Arduino code to demo just about everything!
 * 
 * Actually, it demos traffic lights, NeoPixels and "music".  It's a cobbling together of 
 * three demo programs:
 * - buttoncycler, can't see an author acknowledged
 * - toneMelody, by Tom Igoe: see http://www.arduino.cc/en/Tutorial/Tone
 * - I'm not sure which demo program I used for the traffic lights, but I suspect folk like 
 *     Tom Igoe and Dojo Dave did the original work on the code.
 * My grateful thanks to any and all who provided any of the source code that I started from.
 * 
 * @author   David Argles: d.argles@gmx.com, http://argles.ddns.net/arduino/
 * @version  26-11-2015 @ 17:10h
 */

/*-------------------------------------------------------------------------------------------
 * circuit:
 *  8-ohm speaker on digital pin 8 (plus other side to ground)
 *  red led on pin 13 (plus 470R to ground)
 *  yellow led on pin 12 (plus 470R to ground)
 *  green led on pin 11 (plus 470R to ground)
 *  push-to-make buttons on pins 2, 3 and 4: connect other side of button to 5V, 
 *    plus 10K from pin to ground
 *  NeoPixel string on pin 8 (also ground and 5V)  The notes say you should have 470R at the 
 *    NeoPixel end of the data feed to the string, at least on larger projects.
 * ------------------------------------------------------------------------------------------
 */

#include <Adafruit_NeoPixel.h>  // Need this for the NeoPixel routines
#include "pitches.h"            // This file needs to be in the same "scratch" folder
                                //  It defines the tones for playing music

#define LIGHTS_BUTTON  2    // Digital IO pin for the traffic lights.  This will be
                            // driven with a pull-down resistor so the switch should
                            // pull the pin to 5V momentarily.  On a low -> high
                            // transition the button press logic will execute.
#define NEOPIX_BUTTON  3    // Digital IO pin for the NeoPixel string
#define TUNE_BUTTON    4    // Digital IO pin to start a tune on the speaker

#define RED_PIN       13    // the number of the red LED pin
#define YELLOW_PIN    12    // the number of the yellow LED pin
#define GREEN_PIN     11    // the number of the green LED pin
#define NO_PIN        10    // Nothing attached to pin 10; we use it to turn everything off.
                            //  A bit of a kludge... :-(

#define PIXEL_COUNT   10    // Number of NeoPixels
#define PIXEL_PIN      6    // Digital IO pin connected to the NeoPixels.

// Parameter 1 = number of pixels in strip,  neopixel stick has 8
// Parameter 2 = pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_RGB     Pixels are wired for RGB bitstream
//   NEO_GRB     Pixels are wired for GRB bitstream, correct for neopixel stick
//   NEO_KHZ400  400 KHz bitstream (e.g. FLORA pixels)
//   NEO_KHZ800  800 KHz bitstream (e.g. High Density LED strip), correct for neopixel stick
Adafruit_NeoPixel strip = Adafruit_NeoPixel(PIXEL_COUNT, PIXEL_PIN, NEO_GRB + NEO_KHZ800);

// these are variables that will change
int buttonState = 0;        // variable for reading the pushbutton status
int showType = 0;           // Used to determine which NeoPixel display to give

// melody[] is an array that holds the notes in the melody
//  The 0 is important - it makes a space in the tune, and the tune doesn't sound right without it
int melody[] = {
  NOTE_C4, NOTE_G3, NOTE_G3, NOTE_A3, NOTE_G3, 0, NOTE_B3, NOTE_C4
}
// noteDurations[] is an array that holds the length of the notes in our melody.
//  4 = quarter note, 8 = eighth note, etc.
int noteDurations[] = {
  4, 8, 8, 4, 4, 4, 4, 4
};

// lights[] is an array that holds the light sequence for the traffic lights
//  In this example, NO_PIN effectively means all lights off
int lights[] = {
  GREEN_PIN, YELLOW_PIN, RED_PIN, YELLOW_PIN, NO_PIN, YELLOW_PIN, NO_PIN, YELLOW_PIN, NO_PIN, YELLOW_PIN, NO_PIN, GREEN_PIN, GREEN_PIN
};

// lightDurations[] is an array that holds the light durations in seconds
int lightDurations[] = {
  2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 8
};

/**
 * setup() does all the necessary initial stuff setting up inputs and outputs etc
 */
void setup() {
  // initialize the pushbutton pins as inputs:
  pinMode(LIGHTS_BUTTON, INPUT);
  pinMode(NEOPIX_BUTTON, INPUT);
  pinMode(TUNE_BUTTON, INPUT);
  
  // initialize the LED pins as outputs
  pinMode(RED_PIN, OUTPUT);
  pinMode(YELLOW_PIN, OUTPUT);
  pinMode(GREEN_PIN, OUTPUT);
  pinMode(NO_PIN, OUTPUT);
  // Start with green LED on
  digitalWrite(GREEN_PIN, HIGH);

  // Initialise the NeoPixel strip
  strip.begin();
  strip.show(); // Initialize all pixels to 'off'
}

/**
 * play() plays the tune set up by the melody[] and noteDurations[] arrays
 */
void play() {
  // iterate over the notes of the melody:
  for (int thisNote = 0; thisNote < 8; thisNote++) {

    // to calculate the note duration, take one second
    // divided by the note type.
    //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
    int noteDuration = 1000 / noteDurations[thisNote];
    tone(8, melody[thisNote], noteDuration);

    // to distinguish the notes, set a minimum time between them.
    // the note's duration + 30% seems to work well:
    int pauseBetweenNotes = noteDuration * 1.30;

    delay(pauseBetweenNotes);
    // stop the tone playing:
    noTone(8);
  }
}

/**
 * changelights() progresses through a single cycle of the pedestrian crossing code
 */
void changeLights() {
  // iterate over the notes of the melody:
  for (int thisLight = 0; thisLight < 3; thisLight++) {

    digitalWrite(lights[thisLight-1], LOW);
    digitalWrite(lights[thisLight], HIGH);
    delay(lightDurations[thisLight]*500);
  }
  for (int count=0; count<4; count++) {
    tone(8, NOTE_C8, 125);
    delay(160);
    noTone(8);
    tone(8, NOTE_B7, 125);
    delay(160);
    noTone(8);
  }

  for (int thisLight = 3; thisLight < 12; thisLight++) {
    digitalWrite(lights[thisLight-1], LOW);
    digitalWrite(lights[thisLight], HIGH);
    delay(lightDurations[thisLight]*500);
  }
  //play();
}

/**
 * readButtons() reads the state of the input buttons and 
 * returns a value indicating which button is being pressed
 */
int readButtons()
{
  int result = 0;
  // First we read the buttons
  bool lights = digitalRead(LIGHTS_BUTTON);
  bool neopix = digitalRead(NEOPIX_BUTTON);
  bool tune = digitalRead(TUNE_BUTTON);

  // Now we go through setting result to show which button has been pressed
  //   Note that the first button checked takes precedence
  if(lights) result = LIGHTS_BUTTON;
  else if(neopix) result = NEOPIX_BUTTON;
  else if(tune) result = TUNE_BUTTON;

  return result;
}

/**
 * loop() is the main looping program.  It all happens here!
 */
void loop() {
  // Get current button state.
  //bool buttonState = digitalRead(LIGHTS_BUTTON);
  int buttons = readButtons();
  
  // Loop round checking the buttons.  Drop out as soon as a buton is pressed
  while(buttons==0)buttons = readButtons(); 
  // Now activate the routine corresponding to the pressed button
  switch(buttons)
  {
    case LIGHTS_BUTTON:
      changeLights();
      digitalWrite(GREEN_PIN, HIGH);
      break;
    case NEOPIX_BUTTON:
      showType++;
      if (showType > 9) showType=0;
      startShow(showType);
      break;
    case TUNE_BUTTON:
      play();
      break;
  }
}

/**
 * startshow(i) activates one of ten different NeoPixel displays depending on the value of "i"
 */
void startShow(int i) {
  switch(i){
    case 0: colorWipe(strip.Color(0, 0, 0), 50);    // Black/off
            break;
    case 1: colorWipe(strip.Color(63, 0, 0), 50);  // Red
            break;
    case 2: colorWipe(strip.Color(0, 63, 0), 50);  // Green
            break;
    case 3: colorWipe(strip.Color(0, 0, 63), 50);  // Blue
            break;
    case 4: theaterChase(strip.Color(63, 63, 63), 50); // White
            break;
    case 5: theaterChase(strip.Color(63,   0,   0), 50); // Red
            break;
    case 6: theaterChase(strip.Color(  0,   0, 63), 50); // Blue
            break;
    case 7: rainbow(10);
            break;
    case 8: rainbowCycle(1);
            break;
    case 9: theaterChaseRainbow(5);
            break;
  }
}

// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i, c);
      strip.show();
      delay(wait);
  }
}

void rainbow(uint8_t wait) 
{
  uint16_t i, j;

  for(j=0; j<256; j++) 
  {
    for(i=0; i<strip.numPixels(); i++) 
    {
      strip.setPixelColor(i, Wheel((i+j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256*5; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

//Theatre-style crawling lights.
void theaterChase(uint32_t c, uint8_t wait) 
{
  int span = 4;     // how far between on pixels
  for (int j=0; j<10; j++) {  //do 10 cycles of chasing
    for (int q=0; q < span; q++) {
      for (int i=0; i < strip.numPixels(); i=i+span) {
        strip.setPixelColor(i+q, c);    //turn every third pixel on
      }
      strip.show();
     
      delay(wait);
     
      for (int i=0; i < strip.numPixels(); i=i+span) {
        strip.setPixelColor(i+q, 0);        //turn every third pixel off
      }
    }
  }
}

//Theatre-style crawling lights with rainbow effect
void theaterChaseRainbow(uint8_t wait) {
  int span = 10;     // how far between on pixels
  for (int j=0; j < 256; j++) {     // cycle all 256 colors in the wheel
    for (int q=0; q < span; q++) {
        for (int i=0; i < strip.numPixels(); i=i+span) {
          strip.setPixelColor(i+q, Wheel( (i+j) % 255));    //turn every third pixel on
        }
        strip.show();
       
        delay(wait);
       
        for (int i=0; i < strip.numPixels(); i=i+span) {
          strip.setPixelColor(i+q, 0);        //turn every third pixel off
        }
    }
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {
   return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  } else if(WheelPos < 170) {
    WheelPos -= 85;
   return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  } else {
   WheelPos -= 170;
   return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
  }
}
